// 30.01.2009
// Poszukiwanie turniejw 3-ec na 22 wierzchokach
// graph3ec.cpp zawiera definicje uywanych, a rzadziej lub wcale nie zmienianych funkcji
// Autor: Przemysaw Gordinowicz
// Pomys: autor+Pawe Praat


typedef unsigned char TEdgeStatus;
typedef signed int TVertex;
typedef unsigned long * TEdgePointer;
typedef signed char * TDegreePointer;
typedef TVertex * TPermPointer;
struct TGraph {
    TEdgePointer e_arr;
    TDegreePointer d1, d2;
};
TGraph CreateGraph();
TGraph CreateGraph(TVertex ord);
inline void DestroyGraph(TGraph g);
TGraph GraphCopy(const TGraph g);

struct TGraphListEl {
    TGraph graph;
    TGraphListEl * next; 
};

struct TShortGraphListEl {
	unsigned long e_arr[short_size]; // TEdgePointer 
	TShortGraphListEl * next;
};

struct TShortGraphTreeNode {
	TShortGraphTreeNode * p1, * p2;
};

struct TPermListEl {
	unsigned int nr;
//	TVertex p[order-16];
//  zmiany
	TVertex p[6];
	TPermListEl * next;
};

class THashTable {
public:
    unsigned long hash_size;
    unsigned long data_count;
    unsigned long long comp2_count;

    TGraphListEl ** table;
	char * fname;

    THashTable(const int h_size):
    hash_size(h_size), data_count(0), comp2_count(0), fname(NULL)
    {
        unsigned int i;
        
        table = new TGraphListEl * [hash_size];
        for (i = 0; i < hash_size; i++)
            table[i] = NULL;
    }
    
    ~THashTable() {
		ClearData();
        delete[] table;
    }

    unsigned long Hashing_F(const TGraph g);
    void InsertGraph(const TGraph g);
	void InsertGraphWC(const TGraph g, const unsigned int i);
	void ReadGraphList(const char * filename);
	void WriteGraphList(const char * filename);
	void WriteReport(const char * filename);
	void ClearData();
};

class TSmallHashTable {
public:
    unsigned int hash_size, step;
    unsigned int data_count;
    unsigned short *table;

    TSmallHashTable(const unsigned int h_size, const unsigned int _step):
    hash_size(h_size), step(_step), data_count(0)
    {
        table = new unsigned short [hash_size];
        memset(table, 0, hash_size * sizeof(unsigned short));
    }
    
    ~TSmallHashTable() {
        delete[] table;
    }
	
	void Clear() {
		memset(table, 0, hash_size * sizeof(unsigned short));
		data_count = 0;
	}

    int InsertGraph(const int g_number, const int g_order, const int b, const TVertex * p);
};


//zmienne globalne
THashTable graphs1ec(44);
THashTable * graphs2ec; // inna forma
THashTable graphs10(84), graphs11(672);
TShortGraphTreeNode * graphs10_2ec[84], * graphs11_2ec[672];
int count10[84], count11[672];
unsigned long long test3_count; //tego bywa tak duo !!!
unsigned long long comp_count, cmp2_count; 
unsigned long long degree_count;
unsigned long long read_count, write_count;
unsigned long tree_count;
char fname_in[30], fname_out[30];
unsigned long long pos_min, pos_max, pos_skip;

TGraph StrToGraph(const char * graph_text); // definicja funkcji czciej zmienianej

inline TEdgeStatus GetEdgeStatus(const TEdgePointer p, const TVertex u, const TVertex v) {
	int pos = v;
	pos = (pos * (pos - 1)) / 2 + u;
    return TEdgeStatus((p[pos / 16] >> (2*(pos % 16))) & 3);
}

inline void SetEdgeStatus(TEdgePointer p, const TVertex u, const TVertex v, const TEdgeStatus s) {
	int pos = v;
	pos = (pos * (pos - 1)) / 2 + u;
    p[pos/16] = (p[pos/16] & (~(3 << (2*(pos % 16))))) | (s << (2*(pos % 16)));
}

unsigned long THashTable::Hashing_F(const TGraph g) {
    char i;
	unsigned long long r = g.e_arr[0] % hash_size;
    for(i = 1; i < size; i++)
		r = ((r << 32) + g.e_arr[i]) % hash_size;
	for(i = 0; i < order; i++)
		r = ((((r << 8) + g.d1[i]) << 8) + g.d2[i]) % hash_size;
    return (unsigned long) r;
}

char Compare(const TGraph g1, const TGraph g2) {
    int i;
    for(i = 0; i < size; i++)
		if (g1.e_arr[i] != g2.e_arr[i]) return 0;
	for(i = 0; i < order; i++) {
		if (g1.d1[i] != g2.d1[i]) return 0;
		if (g1.d2[i] != g2.d2[i]) return 0;
	}
    comp_count++;
    return 1;
}

int TSmallHashTable::InsertGraph(const int g_number, const int g_order, const int b, const TVertex * p) {
	unsigned short data = 0;
	TVertex *perm = new TVertex[g_order];
	TVertex *check = new TVertex[g_order];
	TEdgePointer e_arr = graphs1ec.table[g_number]->graph.e_arr;
	TVertex u, v;
	unsigned int pos, st;
	TEdgeStatus s;

	memset(check, 0, g_order * sizeof(TVertex));
	for (u = 0; u < b; u++) {
		perm[u] = p[u];
		check[p[u]] = 1;
	}
	for (v = b, u = 0; u < g_order; u++)
		if (!check[u]) perm[v++] = u;
	
	// obliczenie "macierzy ssiedztwa" grafu
	for (u = 1; u < g_order; u++)
		for (v = 0; v < u; v++) {
			s = (perm[u] < perm[v] ? GetEdgeStatus(e_arr, perm[u], perm[v]) : 3-GetEdgeStatus(e_arr, perm[v], perm[u])) % 2;
			data = (data << 1) + s;
		}
	delete[] check;
	delete[] perm;

	pos = data % hash_size;
	st = data % step + 1;
	while (table[pos] && table[pos] != data)
		pos = (pos + st) % hash_size;
	if (!table[pos]) {
		table[pos] = data;
		data_count++;
		return 1;
	}
	else return 0;
}

TGraph CreateGraph() {
    TGraph g;

    g.e_arr = new unsigned long[size];
    g.d1 = new signed char[order];
    g.d2 = new signed char[order];
    memset(g.e_arr, 0, size * sizeof(unsigned long));
    memset(g.d1, 0, order);
    memset(g.d2, 0, order);
        
    return g;
}

TGraph CreateGraph(TVertex ord) {
    TGraph g;
	int siz = (((ord * (ord-1)) / 2) + 15) / 16;

    g.e_arr = new unsigned long[siz];
    g.d1 = new signed char[ord];
    g.d2 = new signed char[ord];
    memset(g.e_arr, 0, siz * sizeof(unsigned long));
    memset(g.d1, 0, ord);
    memset(g.d2, 0, ord);
        
    return g;
}

inline void DestroyGraph(TGraph g) {
    delete[] g.d2;
    delete[] g.d1;
    delete[] g.e_arr;
}

void CopyGraph(const TGraph src, TGraph dest) {
    int i;
    for(i = 0; i < size; i++)
        dest.e_arr[i] = src.e_arr[i];
    for(i = 0; i < order; i++) {
        dest.d1[i] = src.d1[i];
        dest.d2[i] = src.d2[i];
    }
}

TGraph GraphCopy(const TGraph g) {
    TGraph new_g;

    new_g.e_arr = new unsigned long[size];
    new_g.d1 = new signed char[order];
    new_g.d2 = new signed char[order];
    CopyGraph(g, new_g);

    return new_g;    
}


void GraphToStr(const TGraph g, char * str) {
// tumaczy graf dwustanowy na napis (format Brandana McKey'a) w wersji jednostanowej
// zmiany - jako, ze kodujemy turniej trzeba wybrac wiecej "dodatkow"
//
	TVertex n = 3*order+5;
	TVertex u, v, bit=0, number=0;
	int i;

	if (n < 63) {
	  str[0] = n+63;
	  i = 1;
	}
	else {
		str[0] = 126;
		str[1] = ((n >> 12) & 63) + 63;
		str[2] = ((n >> 6) & 63) + 63;
		str[3] = (n & 63) + 63;
		i = 4;
	}

	// dwa pierwsze wierzchoki (odrnienie I-szej czci) - poczone
	bit=1;
	number=1;
	// I-sza cz ("rda")
	for (u = 0 ; u < order ; u++) {
		for (v = -2 ; v < u ; v++) {
			bit++;
			number <<= 1;
			if (v < 0) 
				number++;
			if (bit == 6) {
				str[i++] = number+63;
				bit = number = 0;
			}
		}
	}
	// II-ga cz (poczenia)
	for (u = order ; u < 2*order ; u++) {
		for (v = -2 ; v < u ; v++) {
			bit++;
			number <<= 1;
			if (v + order == u) 
				number++;
			if (bit == 6) {
				str[i++] = number+63;
				bit = number = 0;
			}
		}
	}
	// III-cia cz (ujcia)
	for (u = 2*order ; u < 3*order ; u++) {
		for (v = -2 ; v < u ; v++) {
			bit++;
			number <<= 1;
			if ((v + order == u) || ((v >= 0) && ((v < u - 2*order && GetEdgeStatus(g.e_arr, v, u-2*order) == 1) ||
								   (v > u - 2*order && v < order && GetEdgeStatus(g.e_arr, u-2*order, v) == 2))))
				number++;
			if (bit == 6) {
				str[i++] = number+63;
				bit = number = 0;
			}
		}
	}

	// u na wszelki wypadek
	u = 3 * order;
	// wierzcholek do rozpoznawania czci III-ciej
	for (v =-2; v < u; v++) {
		bit++;
		number <<= 1;
		if (v >= 2*order)
			number++;
		if (bit == 6) {
			str[i++] = number+63;
			bit = number = 0;
		}
	}
	
	u = 3*order+1;
	// dodatkowy wierzchoek do rozpoznania wyrnionych wierzchokw
	for (v =-2; v < u; v++) {
		bit++;
		number <<= 1;
		if (v >= order && v < 2*order && g.d1[v-order] >= 2*order)
			number++;
		if (bit == 6) {
			str[i++] = number+63;
			bit = number = 0;
		}
	}
	u = 3*order+2;
	// dodatkowy wierzchoek do rozpoznania przetworzonych wierzchokw
	for (v =-2; v < u; v++) {
		bit++;
		number <<= 1;
		if (v >= order && v < 2*order && g.d1[v-order] > 3*order)
			number++;
		if (bit == 6) {
			str[i++] = number+63;
			bit = number = 0;
		}
	}

	if (bit != 0) {
      number <<= (6-bit);
      str[i++] = number+63;
    }
	str[i] = 0;
}

void GraphToStrShort(const TGraph g, char * str) {
// tumaczy graf (krawdzie to 1-ki) na napis (format Brandana McKey'a) 
	TVertex n = order;
	TVertex u, v, bit=0, number=0;
	int i = 1;
	str[0] = n+63;

	for (u = 1 ; u < order ; u++) {
		for (v = 0 ; v < u ; v++) {
			bit++;
			number <<= 1;
			if (GetEdgeStatus(g.e_arr, v, u) == 1)
				number++;
			if (bit == 6) {
				str[i++] = number+63;
				bit = number = 0;
			}
		}
	}
	if (bit != 0) {
      number <<= (6-bit);
      str[i++] = number+63;
    }
	str[i] = 0;
}

TGraph ReadGraph(const char * filename) {
	char str[str_length];
    static FILE * in_file;
	static unsigned long long i;
	static TGraph empty;
	static unsigned long long max_i;

	if (!in_file) {
		in_file = fopen(filename, "rt");
		if (pos_skip > 0) {
			fseeko64(in_file, 0, SEEK_END);
			max_i = ftello64(in_file)/str_length;
			fseeko64(in_file, 0, SEEK_SET);
		}
		if (pos_min > 0) {
			fseeko64(in_file, pos_min*str_length, SEEK_SET);
			i = pos_min;
		}
	}
	
	if (!feof(in_file) && (!pos_max || i <= pos_max)) {
		fscanf(in_file, "%s\n", str);
		i++;
		if (pos_skip) {
			i += pos_skip;
			if (i >= max_i) {
				i = max_i;
				pos_skip = 0;
				fseeko64(in_file, 0, SEEK_END);
				fscanf(in_file, "\n");
			}
			if (!feof(in_file)) fseeko64(in_file, i*str_length, SEEK_SET);
		}
		read_count++;
		return StrToGraph(str);
	}
	else {
		fclose(in_file);
		return empty;
	}
}

void EmbedGraph(const TGraph src, TGraph dest, const TVertex src_order, const TPermPointer place) {
// osadza graf src w grafie dest na wierzchokach (kolejnych)
// okrelonych przez place[v]
// zakada, e zmieniane krawdzie grafu dest byy na tych
// wierzchokach nieokrelone (0)
// zmiany - to jest turniej - antysymetria
    TVertex u, v;
	TEdgeStatus st;
    for(u = 0; u < src_order; u++) {
		for(v = u+1; v < src_order; v++) { 
			st = GetEdgeStatus(src.e_arr, u, v);
            if (place[u] < place[v])
                SetEdgeStatus(dest.e_arr, place[u], place[v], st);
            else SetEdgeStatus(dest.e_arr, place[v], place[u], (3 - st) % 3);
			switch (st) {
				case 1: dest.d1[place[u]]++;
						dest.d2[place[v]]++;
						break;
				case 2: dest.d2[place[u]]++;
						dest.d1[place[v]]++;
			}
		}
    }
}

void THashTable::InsertGraph(const TGraph g) {
    //if (hash_size == 1)
	//	InsertGraphWC(g, 0);
	//else 
	{
		const unsigned long i = Hashing_F(g);
		TGraphListEl * w = table[i];
	    
		if (!w) {
			table[i] = new TGraphListEl;
			table[i]->graph = GraphCopy(g);
			table[i]->next = NULL;
			data_count++;
		}
		else {
			while (w->next && !Compare(g, w->graph))
				w = w->next;
			if (!w->next && !Compare(g, w->graph)) {
				w->next = new TGraphListEl;
				w->next->graph = GraphCopy(g);
				w->next->next = NULL;
				data_count++;    
			}
			else comp2_count++;
		}
	}
	if (fname && data_count > 2 * max_hash_size) { // wyrzu dane na dysk i zwolnij pami
		printf("\tZapis\t");
		fflush(stdout);
		WriteGraphList(fname);
		ClearData();
	}
}

void THashTable::InsertGraphWC(const TGraph g, const unsigned int i) {
    TGraphListEl * w = table[i];
    
    table[i] = new TGraphListEl;
    table[i]->graph = GraphCopy(g);
    table[i]->next = w;
    data_count++;

	if (fname && data_count > max_hash_size) { // wyrzu dane na dysk i zwolnij pami (celowo 2x czciej)
		printf("\tZapis\t");
		fflush(stdout);
		WriteGraphList(fname);
		ClearData();
	}
}


void PrintGraph(TGraph g) {
    int j;
    for(j = 0; j < size-1; j++)
        printf("%d ", g.e_arr[j]);
    printf("%d\n", g.e_arr[size-1]);
    for(j = 0; j < order-1; j++)
        printf("%d ", g.d1[j]);
    printf("%d\n", g.d1[order-1]);
    for(j = 0; j < order-1; j++)
        printf("%d ", g.d2[j]);
    printf("%d\n", g.d2[order-1]);    
}

void THashTable::ReadGraphList(const char * filename) {
// aktualnie nieuywana funkcja
	TGraphListEl * w;
	char str[str_length];
    FILE * in_file = fopen(filename, "rt");
	unsigned int i=0;

	read_count -= data_count;
	while (!feof(in_file)) {
		fscanf(in_file, "%s\n", str);
		i++;
		if (i > pos_min && (!pos_max || i <= pos_max) && !((i-1-pos_min) % (pos_skip + 1))) {
			w = table[0];
			table[0] = new TGraphListEl;
			table[0]->next = w;
			table[0]->graph = StrToGraph(str);
			data_count++;
		}
	}
    fclose(in_file);
	read_count += data_count;
}

void THashTable::WriteGraphList(const char * filename) {
    unsigned long i;
    TGraphListEl * w;
	char str[str_length];
    FILE * out_file = fopen(filename, "at");

    for(i = 0, w = table[i]; i < hash_size; w = table[++i])
        while (w) {
			GraphToStr(w->graph, str);
            fprintf(out_file, "%s\n", str);
            w = w->next;
        }
    fclose(out_file);
	write_count += data_count;
}

void THashTable::WriteReport(const char * filename) {
    unsigned long i;
    TGraphListEl * w;
	char str[str_length];
    FILE * out_file = fopen(filename, "at");

    for(i = 0, w = table[i]; i < hash_size; w = table[++i])
        while (w) {
			GraphToStrShort(w->graph, str);
            fprintf(out_file, "%s\n", str);
            w = w->next;
        }
	fprintf(out_file, "A\n");
    fclose(out_file);
}

void THashTable::ClearData() {
	unsigned int i;
	TGraphListEl * w;
	for(i = 0; i < hash_size; i++) {
		while (table[i]) {
			w = table[i];
			table[i] = table[i]->next;
			DestroyGraph(w->graph);
			delete w;
		}
		table[i] = NULL;
	}
	data_count = 0;
}

int Test3ec(const TGraph g) {
// sprawdza warunki 3-ec dla grafu g (3-stanowego)
// zwraca 0 - graf nie moe tworzy grafu 3-ec
// lub 1 - nie mona tego wykluczy
// mona dorobi - uzupenianie 0 (znakw zapytania)
// konkretnymi wartociami - gdy s jednoznaczne
// zmiany - trzeba sprawdza na obie strony (antysymetria)

	unsigned long nonsucc[order], nonpred[order], test;
	TVertex x, y, z;
	
	for (x = 0; x < order; x++)
		nonsucc[x] = nonpred[x] = 0;

	// zapisanie grafu w "odpowiedniej" tablicy bitowej
	for (x = 0; x < order-1; x++)
		for (y = x+1; y < order; y++)
			switch (GetEdgeStatus(g.e_arr, x, y)) {
				case 0: nonsucc[x] |= (1 << y);  // robi 0 i 1
						nonpred[y] |= (1 << x);
				case 1: nonpred[x] |= (1 << y);
						nonsucc[y] |= (1 << x);
						break;
				default: nonsucc[x] |= (1 << y); // case 2
						 nonpred[y] |= (1 << x);
		}

	// test3-ec waciwy
	for (x = 0; x < order-2; x++)
		for (y = x+1; y < order-1; y++)
			for (z = y+1; z < order; z++) {
				test = (nonsucc[x] & nonsucc[y] & nonsucc[z]) && (nonsucc[x] & nonsucc[y] & nonpred[z]) &&
					   (nonsucc[x] & nonpred[y] & nonsucc[z]) && (nonsucc[x] & nonpred[y] & nonpred[z]) &&
					   (nonpred[x] & nonsucc[y] & nonsucc[z]) && (nonpred[x] & nonsucc[y] & nonpred[z]) &&
					   (nonpred[x] & nonpred[y] & nonsucc[z]) && (nonpred[x] & nonpred[y] & nonpred[z]);
				if (!test) {
					test3_count++;
					return 0;
				}
			}
	// jak tu jestemy, to znaczy, ze test przeszed
	return 1;
}

void EmbedIsoGraph9(TGraph dest, const TVertex vert, const TEdgeStatus st, const TVertex z0, const TVertex z1, const TVertex zf, 
                     const TPermPointer p, THashTable * table, int g_start=1, int g_stop=14) {
// osadza (po kolei) wszystkie turnieje izomorficzne z T_9
// w grafie dest na wierzchokach okrelonych przez (p1[v] v=0..z1-1)
// i dobraniu pozostaych wierzchokw z p1[v] v=z1..z1+z0-1
// o ile to jest moliwe (zmieniane s tylko krawdzie nieokrelone (0)
// wierzchoki z p0 ktre wchodz do grafu na poczeniu z wierzchokiem
// vert dostaja status st a te ktre nie wchodz status (st mod 2) + 1
// prbuj obsuy rwnie z1 == 0 (nie testowane)
// mozna wziac tylko wybrane turnieje T_9 (miedzy start i stop)
    if (z1 + z0 >= 9) {
        TGraph g = CreateGraph();
        TVertex a, u, v, x, y;
        char q;
        TEdgeStatus s1, s2;
        int i;
        
        TPermPointer place = new TVertex[order];
        TPermPointer rev = new TVertex[order];
		for(u = 0; u < order; u++) 
			place[u] = rev[u] = -1;

		for (i = g_start; i <= g_stop; i++) {
            TGraph src = graphs2ec->table[i]->graph;

            place[0] = p[0];
            rev[0] = 0;
            a = 1;
            do {        // rozoy wierzchoki z p1 i p0 w place tak eby byo ok
                while ((a < z1) && (rev[0] != -1)) {
                    if (rev[a] != -1) place[rev[a]] = -1;
                    u = rev[a] + 1;
                    do {
                        while (place[u] != -1) u++;
                        // czy p[a] moe by na pozycji u?
                        q = 0;
                        if (u < 9)
							for (v = 0; v < a; v++) {
								s1 = u < rev[v] ? GetEdgeStatus(src.e_arr, u, rev[v]) : 3-GetEdgeStatus(src.e_arr, rev[v], u);
								s2 = p[a] < p[v] ? GetEdgeStatus(dest.e_arr, p[a], p[v]) : (3-GetEdgeStatus(dest.e_arr, p[v], p[a]))%3;
								q = (s2 > 0) && (s1 != s2);
								if (q) break;
							}
                        if (q) u++;    
                    } while (q && (u < 9));
                    if (u >= 9) {
                        rev[a] = -1;
                        a--;
                    }
                    else {
                        place[u] = p[a];
                        rev[a] = u;
                        a++;
                    }
                }   

                if (rev[0] != -1) {      // dobrano wierzchoki "obowizkowe" - czas na potencjalne
                    if (z0 > 0) do {
                        if (rev[a] != -1) place[rev[a]] = -1;
                        u = rev[a] + 1;
                        do {
                            while (place[u] != -1) u++;
                            // czy p[a] moe by na pozycji u ?
                            q = 0;
                            if (u < 9)
                                for (v = 0; v < 9; v++)
                                    if (place[v] != -1) {
                                        s1 = u < v ? GetEdgeStatus(src.e_arr, u, v) : 3-GetEdgeStatus(src.e_arr, v, u);
                                        s2 = p[a] < place[v] ? GetEdgeStatus(dest.e_arr, p[a], place[v]) : (3-GetEdgeStatus(dest.e_arr, place[v], p[a]))%3;
                                        q = (s2 > 0) && (s1 != s2);
                                        if (q) break;
                                    }
                            if (q) u++;    
                        } while (q && (u < 10) && ((u != 9) || (place[9] != -1)));
                        if (u >= z0 + z1) {
                            rev[a] = -1;
                            a--;
                            while (a >= z0+z1-zf || (a >= 0 && rev[a] >= 9)) {
                                place[rev[a]] = -1;
                                rev[a] = -1;
                                a--;
                            }
                        }
                        else {
                            place[u] = p[a];
                            rev[a] = u;
                            a++; 
                        }
                    } while ((a < z0 + z1) && (rev[z1] != -1));
                    if (a >= z0 + z1) a = z0 + z1 -1;
                    if (z0 == 0 || rev[z1] != -1) {
                        // Wstawi graf
                        CopyGraph(dest, g);
                        q = 0;
                        for(u = 0; (u < 8) && !q; u++) {
                            for(v = u+1; (v < 9) && !q; v++) {
                                if (place[u] < place[v]) {
                                    x = place[u];
                                    y = place[v];
                                }
                                else {
                                    x = place[v];
                                    y = place[u];                            
                                }
								if (!GetEdgeStatus(g.e_arr, x, y)) {
									s1 = GetEdgeStatus(src.e_arr, u, v);
									if (place[v] < place[u]) s1 = 3 - s1;
									SetEdgeStatus(g.e_arr, x, y, s1);
									// zostaje order - 10 bo ju wiem, ze 7 odpada
									q = (s1 == 1) ? (((++(g.d1[x])%order) > order - 10) || ((++(g.d2[y])%order) > order - 10)) : 
													(((++(g.d2[x])%order) > order - 10) || ((++(g.d1[y])%order) > order - 10));
								}
							}
						}
						if (!q) {
							for(v = 0; (v < z0) && !q; v++) {
								s1 = rev[z1+v] < 9 ? st : 3-st;
								if (vert < p[z1+v])
									SetEdgeStatus(g.e_arr, vert, p[z1+v], s1);
								else SetEdgeStatus(g.e_arr, p[z1+v], vert, 3-s1);
								q = (s1 == 1) ? (++(g.d1[vert]),(++(g.d2[p[z1+v]])%order) > order - 10) : 
												(++(g.d2[vert]),(++(g.d1[p[z1+v]])%order) > order - 10);
							}
                            if (!q) {
								if (Test3ec(g)) table->InsertGraph(g);
                            }
                        }
					}
                }
            } while (rev[0] != -1);
        }
        DestroyGraph(g);
        delete[] rev;
        delete[] place;
    }
}


void EmbedIsoGraph10(TGraph dest, const TVertex vert, const TEdgeStatus st, const TVertex z0, const TVertex z1, const TVertex zf, 
                     const TPermPointer p, THashTable * table, int g_start=15, int g_stop=1097) {
// osadza (po kolei) wszystkie turnieje izomorficzne z T_10
// w grafie dest na wierzchokach okrelonych przez (p1[v] v=0..z1-1)
// i dobraniu pozostaych wierzchokw z p1[v] v=z1..z1+z0-1
// o ile to jest moliwe (zmieniane s tylko krawdzie nieokrelone (0)
// wierzchoki z p0 ktre wchodz do grafu na poczeniu z wierzchokiem
// vert dostaja status st a te ktre nie wchodz status (st mod 2) + 1
// prbuj obsuy rwnie z1 == 0 (nie testowane)
// mozna wziac tylko wybrane turnieje T_9 (miedzy start i stop)
    if (z1 + z0 >= 10) {
        TGraph g = CreateGraph();
        TVertex a, u, v, x, y;
        char q;
        TEdgeStatus s1, s2;
        int i;
        
        TPermPointer place = new TVertex[order];
        TPermPointer rev = new TVertex[order];
		for(u = 0; u < order; u++) 
			place[u] = rev[u] = -1;

		for (i = g_start; i <= g_stop; i++) {
            TGraph src = graphs2ec->table[i]->graph;

            place[0] = p[0];
            rev[0] = 0;
            a = 1;
            do {        // rozoy wierzchoki z p1 i p0 w place tak eby byo ok
                while ((a < z1) && (rev[0] != -1)) {
                    if (rev[a] != -1) place[rev[a]] = -1;
                    u = rev[a] + 1;
                    do {
                        while (place[u] != -1) u++;
                        // czy p[a] moe by na pozycji u?
                        q = 0;
                        if (u < 10)
							for (v = 0; v < a; v++) {
								s1 = u < rev[v] ? GetEdgeStatus(src.e_arr, u, rev[v]) : 3-GetEdgeStatus(src.e_arr, rev[v], u);
								s2 = p[a] < p[v] ? GetEdgeStatus(dest.e_arr, p[a], p[v]) : (3-GetEdgeStatus(dest.e_arr, p[v], p[a]))%3;
								q = (s2 > 0) && (s1 != s2);
								if (q) break;
							}
                        if (q) u++;    
                    } while (q && (u < 10));
                    if (u >= 10) {
                        rev[a] = -1;
                        a--;
                    }
                    else {
                        place[u] = p[a];
                        rev[a] = u;
                        a++;
                    }
                }   

                if (rev[0] != -1) {      // dobrano wierzchoki "obowizkowe" - czas na potencjalne
                    if (z0 > 0) do {
                        if (rev[a] != -1) place[rev[a]] = -1;
                        u = rev[a] + 1;
                        do {
                            while (place[u] != -1) u++;
                            // czy p[a] moe by na pozycji u ?
                            q = 0;
                            if (u < 10)
                                for (v = 0; v < 10; v++)
                                    if (place[v] != -1) {
                                        s1 = u < v ? GetEdgeStatus(src.e_arr, u, v) : 3-GetEdgeStatus(src.e_arr, v, u);
                                        s2 = p[a] < place[v] ? GetEdgeStatus(dest.e_arr, p[a], place[v]) : (3-GetEdgeStatus(dest.e_arr, place[v], p[a]))%3;
                                        q = (s2 > 0) && (s1 != s2);
                                        if (q) break;
                                    }
                            if (q) u++;    
                        } while (q && (u < 11) && ((u != 10) || (place[10] != -1)));
                        if (u >= z0 + z1) {
                            rev[a] = -1;
                            a--;
                            while (a >= z0+z1-zf || (a >= 0 && rev[a] >= 10)) {
                                place[rev[a]] = -1;
                                rev[a] = -1;
                                a--;
                            }
                        }
                        else {
                            place[u] = p[a];
                            rev[a] = u;
                            a++; 
                        }
                    } while ((a < z0 + z1) && (rev[z1] != -1));
                    if (a >= z0 + z1) a = z0 + z1 -1;
                    if (z0 == 0 || rev[z1] != -1) {
                        // Wstawi graf
                        CopyGraph(dest, g);
                        q = 0;
                        for(u = 0; (u < 9) && !q; u++) {
                            for(v = u+1; (v < 10) && !q; v++) {
                                if (place[u] < place[v]) {
                                    x = place[u];
                                    y = place[v];
                                }
                                else {
                                    x = place[v];
                                    y = place[u];                            
                                }
								if (!GetEdgeStatus(g.e_arr, x, y)) {
									s1 = GetEdgeStatus(src.e_arr, u, v);
									if (place[v] < place[u]) s1 = 3 - s1;
									SetEdgeStatus(g.e_arr, x, y, s1);
									// zostaje order - 10 bo ju wiem, ze 7 odpada
									q = (s1 == 1) ? (((++(g.d1[x])%order) > order - 10) || ((++(g.d2[y])%order) > order - 10)) : 
													(((++(g.d2[x])%order) > order - 10) || ((++(g.d1[y])%order) > order - 10));
								}
							}
						}
						if (!q) {
							for(v = 0; (v < z0) && !q; v++) {
								s1 = rev[z1+v] < 10 ? st : 3-st;
								if (vert < p[z1+v])
									SetEdgeStatus(g.e_arr, vert, p[z1+v], s1);
								else SetEdgeStatus(g.e_arr, p[z1+v], vert, 3-s1);
								q = (s1 == 1) ? (++(g.d1[vert]),(++(g.d2[p[z1+v]])%order) > order - 10) : 
												(++(g.d2[vert]),(++(g.d1[p[z1+v]])%order) > order - 10);
							}
                            if (!q) {
								if (Test3ec(g)) 
									table->InsertGraph(g);
                            }
                        }
					}
                }
            } while (rev[0] != -1);
        }
        DestroyGraph(g);
        delete[] rev;
        delete[] place;
    }
}



void EmbedIsoGraph10p(TGraph dest, const TVertex vert, const TEdgeStatus st, const TVertex z0, const TVertex z1, const TVertex zf, 
					 const TPermPointer p, THashTable * table, int full = 1721, int g_start = 0, int g_stop = 84) {
// zmiany(turniej)
// osadza (po kolei) wszystkie turnieje izomorficzne z kolejnymi T_10
// (niedookrelonymi kandydatami na 2-ec - jeden wierzchoek peny dwa 
// grafy 1-ec w jego otoczeniu i przeciwotoczeniu
// w grafie dest na wierzchokach okrelonych przez (p1[v] v=0..z1-1)
// i dobraniu pozostaych wierzchokw z p1[v] v=z1..z1+z0-1
// o ile to jest moliwe (zmieniane s tylko krawdzie nieokrelone (0)
// wierzchoki z p0 ktre wchodz do grafu na poczeniu z wierzchokiem
// vert dostaja status st a te ktre nie wchodz status (st mod 2) + 1
// wana z1 > 0 !
// mozna wziac tylko wybrane grafy T_10 (miedzy start i stop)
    if (z1 + z0 >= 10) {
		int adr[5] = {0, 1, 2, 8, 44};
		TGraph g = CreateGraph(), src;
        TVertex a, u, v, x, y;
		TVertex ord, b1, b2;
        char q;
        TEdgeStatus s1, s2;
        int i;
        TPermPointer place = new TVertex[order];
        TPermPointer rev = new TVertex[order];
		TDegreePointer b = new signed char[order];
		for(u = 0; u < order; u++) {
			place[u] = rev[u] = -1;
			b[u] = 0;
		}
		TPermPointer p1 = new TVertex[order];
		for(u = 0; u < z1+z0; u++) {
			p1[u] = p[u];
		}
		TPermListEl * list1[4] = {NULL, NULL, NULL, NULL}, * list2[4] = {NULL, NULL, NULL, NULL}, *w;
		THashTable *temp = new THashTable(29999);
		TSmallHashTable hash_table(1399, 37);
		unsigned int dc;

		b1 = b2 = 0;
		for (v = 1; v < z1; v++)
			switch (p1[0] < p1[v] ? GetEdgeStatus(dest.e_arr, p1[0], p1[v]) : 3-GetEdgeStatus(dest.e_arr, p1[v], p1[0])) {
				case 1: b1++; b[p1[v]] = 2; break;
				case 2: b2++; b[p1[v]] = 1;
			}
		//b1 = 0;
		//b2 = 0;
		//posortowa p1[1..z1] dodatkowo wedug b - insertion sort
		for (v = 2; v < z1; v++) { // cig 1..v-1 jest ju posortowany
			x = p1[v];
			for (u = v-1; u >= 1 && b[x] > b[p1[u]]; u--)
				p1[u+1] = p1[u];
			p1[u+1] = x;
		}
		
		for (ord = b1; ord < 10-b2; ord++) if (ord >= 3 && ord <= 6) {
			//otoczenie
			for (i = adr[ord-3]; i < adr[ord-2]; i++) {
				src = graphs1ec.table[i]->graph;
				if (b1 >= 1) {
					hash_table.Clear();
					place[0] = p1[1];
					rev[1] = 0;
					a = 2;
					do {        // rozoy wierzchoki z p1[1..b1] w place tak eby byo ok
						while ((a <= b1) && (rev[1] != -1)) {
							if (rev[a] != -1) place[rev[a]] = -1;
							u = rev[a] + 1;
							do {
								while (place[u] != -1) u++;
								// czy p1[a] moe by na pozycji u?
								q = 0;
								if (u < ord)
									for (v = 1; v < a; v++) {
										s1 = u < rev[v] ? GetEdgeStatus(src.e_arr, u, rev[v]) : 3-GetEdgeStatus(src.e_arr, rev[v], u);
										s2 = p1[a] < p1[v] ? GetEdgeStatus(dest.e_arr, p1[a], p1[v]) : (3-GetEdgeStatus(dest.e_arr, p1[v], p1[a])) % 3;
										q = (s2 > 0) && (s1 != s2);
										if (q) break;
									}
								if (q) u++;    
							} while (q && (u < ord));
							if (u >= ord) {
								rev[a] = -1;
								a--;
							}
							else {
								place[u] = p1[a];
								rev[a] = u;
								a++;
							}
						}
						if (a > b1) a = b1;
						if (rev[1] != -1) {      // dobrano wierzchoki "obowizkowe" - wrzucamy na list do przetworzenia
							w = new TPermListEl;
							w->next = list1[ord-3];
							w->nr = i;
							for (u = 1; u <= b1; u++)
								w->p[u-1] = rev[u];
							if (hash_table.InsertGraph(i, ord, b1, w->p))
								list1[ord-3] = w;
							else delete w;
						}
					} while (rev[1] != -1);
				}
				else { // wstawi pusty ukad;
					w = new TPermListEl;
					w->next = list1[ord-3];
					w->nr = i;
					list1[ord-3] = w;
				}
			}
			//przeciwotoczenie graf rzedu 9-ord
			for (i = adr[6-ord]; i < adr[7-ord]; i++) {
				src = graphs1ec.table[i]->graph;
				if (b2 >= 1) {
					hash_table.Clear();
					place[0] = p1[b1+1];
					rev[b1+1] = 0;
					a = b1+2;
					do {        // rozoy wierzchoki z p1[b1+1..b1+b2] w place tak eby byo ok
						while ((a <= b1+b2) && (rev[b1+1] != -1)) {
							if (rev[a] != -1) place[rev[a]] = -1;
							u = rev[a] + 1;
							do {
								while (place[u] != -1) u++;
								// czy p1[a] moe by na pozycji u?
								q = 0;
								if (u < 9-ord)
									for (v = b1+1; v < a; v++) {
										s1 = u < rev[v] ? GetEdgeStatus(src.e_arr, u, rev[v]) : 3-GetEdgeStatus(src.e_arr, rev[v], u);
										s2 = p1[a] < p1[v] ? GetEdgeStatus(dest.e_arr, p1[a], p1[v]) : (3-GetEdgeStatus(dest.e_arr, p1[v], p1[a])) % 3;
										q = (s2 > 0) && (s1 != s2);
										if (q) break;
									}
								if (q) u++;    
							} while (q && (u < 9-ord));
							if (u >= 9-ord) {
								rev[a] = -1;
								a--;
							}
							else {
								place[u] = p1[a];
								rev[a] = u;
								a++;
							}
						}   
						if (a > b1+b2) a = b1+b2;
						if (rev[b1+1] != -1) {      // dobrano wierzchoki "obowizkowe" - wrzucamy na list do przetworzenia
							w = new TPermListEl;
							w->next = list2[6-ord];
							w->nr = i;
							for (u = 0; u < b2; u++)
								w->p[u] = rev[u+b1+1];
							if (hash_table.InsertGraph(i, 9-ord, b2, w->p))
								list2[6-ord] = w;
							else delete w;
						}
					} while (rev[b1+1] != -1);
				}
				else { // wstawi pusty ukad;
					w = new TPermListEl;
					w->next = list2[6-ord];
					w->nr = i;
					list2[6-ord] = w;
				}
			}
		}
		
		place[0] = p1[0];
		rev[0] = 0;
		for (ord = 3; ord <= 6; ord++) {
			while (list1[ord-3]) {
				for (u = 1; u <= b1; u++) {
					rev[u] = list1[ord-3]->p[u-1]+1;
					place[rev[u]] = p1[u];
				}
				for (w = list2[6-ord]; w; w = w->next) {
					i = 0;
					for (a = 3; a < ord; a++)
						i += (adr[a-2] - adr[a-3]) * (adr[7-a] - adr[6-a]);
					i += (list1[ord-3]->nr - adr[ord-3]) * (adr[7-ord] - adr[6-ord]);
					i += w->nr - adr[6-ord];
					if (i >= g_start && i <= g_stop && graphs10.table[i]) {
						src = graphs10.table[i]->graph;
						if (!b2) {
							if (!b1) {
								place[0] = p1[0];
								rev[0] = 0;
							}
							else {
								rev[b1] = list1[ord-3]->p[b1-1]+1;
								place[rev[b1]] = p1[b1];
							}
						}
						for (u = b1+1; u <= b1+b2; u++) {
							rev[u] = w->p[u-b1-1]+ord+1;
							place[rev[u]] = p1[u];
						}
						a = b1 + b2 + 1;
						//tutaj reszta
						do {        // rozoy wierzchoki z p1 i p0 w place tak eby byo ok
							while ((a < z1) && (rev[b1+b2] != -1)) {
								if (rev[a] != -1) place[rev[a]] = -1;
								u = rev[a] + 1;
								do {
									while (place[u] != -1) u++;
									// czy p1[a] moe by na pozycji u?
									q = 0;
									if (u < 10)
										for (v = 0; v < a; v++) {
											s1 = u < rev[v] ? GetEdgeStatus(src.e_arr, u, rev[v]) : (3-GetEdgeStatus(src.e_arr, rev[v], u)) % 3;
											s2 = p1[a] < p1[v] ? GetEdgeStatus(dest.e_arr, p1[a], p1[v]) : (3-GetEdgeStatus(dest.e_arr, p1[v], p1[a])) % 3;
											q = (s1 > 0) && (s2 > 0) && (s1 != s2);
											if (q) break;
										}
									if (q) u++;    
								} while (q && (u < 10));
								if (u >= 10) {
									rev[a] = -1;
									a--;
									if (a == b1+b2) {
										place[rev[b1+b2]] = -1;
										rev[b1+b2] = -1; // koniec - bo wierzchokw poniej b1+b2 nie ruszamy
									}
								}
								else {
									place[u] = p1[a];
									rev[a] = u;
									a++;
								}
							}   

							if (rev[b1+b2] != -1) {      // dobrano wierzchoki "obowizkowe" - czas na potencjalne
								if (z0 > 0) do {
									if (rev[a] != -1) place[rev[a]] = -1;
									u = rev[a] + 1;
									do {
										while (place[u] != -1) u++;
										// czy p1[a] moe by na pozycji u ?
										q = 0;
										if (u < 10)
											for (v = 0; v < 10; v++)
												if (place[v] != -1) {
													s1 = u < v ? GetEdgeStatus(src.e_arr, u, v) : (3-GetEdgeStatus(src.e_arr, v, u)) % 3;
													s2 = p1[a] < place[v] ? GetEdgeStatus(dest.e_arr, p1[a], place[v]) : (3-GetEdgeStatus(dest.e_arr, place[v], p1[a])) % 3;
													q = (s1 > 0) && (s2 > 0) && (s1 != s2);
													if (q) break;
												}
										if (q) u++;    
									} while (q && (u < 11) && ((u != 10) || (place[10] != -1)));
									if (u >= z0 + z1) {
										rev[a] = -1;
										a--;
										while (a >= z0+z1-zf || (a >= 0 && rev[a] >= 10)) {
											place[rev[a]] = -1;
											rev[a] = -1;
											a--;
										}
									}
									else {
										place[u] = p1[a];
										rev[a] = u;
										a++; 
									}
								} while ((a < z0 + z1) && (rev[z1] != -1));
								if (a >= z0 + z1) a = z0 + z1 -1;
								if (z0 == 0 || rev[z1] != -1) {
									// Wstawi graf
									CopyGraph(dest, g);
									q = 0;
									for(u = 0; (u < 9) && !q; u++) {
										for(v = u+1; (v < 10) && !q; v++) {
											if (place[u] < place[v]) {
												x = place[u];
												y = place[v];
											}
											else {
												x = place[v];
												y = place[u];                            
											}
											if (!GetEdgeStatus(g.e_arr, x, y)) {
												if (s1 = GetEdgeStatus(src.e_arr, u, v)) {
													if (place[v] < place[u]) s1 = 3 - s1;
													SetEdgeStatus(g.e_arr, x, y, s1);
													q = (s1 == 1) ? (((++(g.d1[x])%order) > order - 10) || ((++(g.d2[y])%order) > order - 10)) :
																	(((++(g.d2[x])%order) > order - 10) || ((++(g.d1[y])%order) > order - 10));
												}
											}
										}
									}
									if (!q) {
										for(v = 0; (v < z0) && !q; v++) {
											s1 = rev[z1+v] < 10 ? st : 3 - st;
											if (vert < p1[z1+v])
												SetEdgeStatus(g.e_arr, vert, p1[z1+v], s1);
											else SetEdgeStatus(g.e_arr, p1[z1+v], vert, 3 - s1);
											q = (s1 == 1) ? (++(g.d1[vert]),(++(g.d1[p1[z1+v]])%order) > order - 10) : 
															(++(g.d2[vert]),(++(g.d2[p1[z1+v]])%order) > order - 10);
										}
										if (!q) {
											if (full > count10[i]) { // przejrzec list grafw - pniej w nieco atwiejszej wersji
												dc = temp->data_count; // mona mniej
												temp->InsertGraph(g);
												if (temp->data_count > dc) {
													TShortGraphTreeNode * w2 = graphs10_2ec[i];
													TVertex v_stack[60];
													TShortGraphTreeNode * t_stack[20];
													int stack_count = 1;
													u = 1; v = ord+1;
													while (stack_count) {
														stack_count--;
														q = 0;
														for( ; (u <= ord) && !q; u++, v = ord+1) {
															for( ; (v < 10) && !q; v++) {
																if (place[u] < place[v])  s1 = GetEdgeStatus(g.e_arr, place[u], place[v]); 
																else s1 = (3-GetEdgeStatus(g.e_arr, place[v], place[u])) % 3; 
																switch (s1) {
																	case 1: w2 = w2->p1; break;
																	case 2: w2 = w2->p2; break;
																	default : if (w2->p1) {
																				if (place[u] < place[v]) SetEdgeStatus(g.e_arr, place[u], place[v], 1);
																				else SetEdgeStatus(g.e_arr, place[v], place[u], 2);
																				g.d1[place[u]]++;
																				g.d2[place[v]]++;
																				q = ((g.d1[place[u]] % order) > order - 10) || ((g.d2[place[v]] % order) > order - 10);
																				v_stack[3*stack_count] = u;
																				v_stack[3*stack_count+1] = v;
																				v_stack[3*stack_count+2] = 1;
																				t_stack[stack_count++] = w2;
																				w2 = w2->p1;
																			  }
																			  else { //w2->p2
																				if (place[u] < place[v]) SetEdgeStatus(g.e_arr, place[u], place[v], 2);
																				else SetEdgeStatus(g.e_arr, place[v], place[u], 1);
																				g.d2[place[u]]++;
																				g.d1[place[v]]++;
																				q = ((g.d2[place[u]] % order) > order - 10) || ((g.d1[place[v]] % order) > order - 10);
																				v_stack[3*stack_count] = u;
																				v_stack[3*stack_count+1] = v;
																				v_stack[3*stack_count+2] = 2;
																				t_stack[stack_count++] = w2;
																				w2 = w2->p2;
																			  }
																}
																q = q || !w2;
															}
														}
														if (!q && Test3ec(g)) 
															table->InsertGraph(g);
														q = 1;
														while (q) 
															if (stack_count) {
																u = v_stack[3*stack_count-3];
																v = v_stack[3*stack_count-2];
																s1 = v_stack[3*stack_count-1];
																w2 = t_stack[stack_count-1];
																if (s1 == 1) {
																	g.d1[place[u]]--;
																	g.d2[place[v]]--;
																	if (w2->p2) {
																		if (place[u] < place[v]) SetEdgeStatus(g.e_arr, place[u], place[v], 2);
																		else SetEdgeStatus(g.e_arr, place[v], place[u], 1);
																		g.d2[place[u]]++;
																		g.d1[place[v]]++;
																		q = ((g.d2[place[u]] % order) > order - 10) || ((g.d1[place[v]] % order) > order - 10);
																		if (q) {
																			g.d2[place[u]]--;
																			g.d1[place[v]]--;
																			if (place[u] < place[v]) SetEdgeStatus(g.e_arr, place[u], place[v], 0);
																		    else SetEdgeStatus(g.e_arr, place[v], place[u], 0);
																		}
																		else {
																			v_stack[3*stack_count-1] = 2;
																			stack_count++;
																			v++;
																			w2=w2->p2;
																		}
																	}
																	else if (place[u] < place[v]) SetEdgeStatus(g.e_arr, place[u], place[v], 0);
																		    else SetEdgeStatus(g.e_arr, place[v], place[u], 0);
																}
																else { //s1 == 2
																	g.d2[place[u]]--;
																	g.d1[place[v]]--;
																	if (place[u] < place[v]) SetEdgeStatus(g.e_arr, place[u], place[v], 0);
																    else SetEdgeStatus(g.e_arr, place[v], place[u], 0);
																}
																if (q) stack_count--;
															}
															else q = 0;
													}
												}
											}
											else if (Test3ec(g)) table->InsertGraph(g);  
										}
									}
								}
							}
						} while (rev[b1+b2] != -1);
						
						for (u = b1+1; u < b1+b2; u++) {
							place[rev[u]] = -1;
							rev[u] = -1;
						}
					}
				}
				w = list1[ord-3];
				list1[ord-3] = w->next;
				if (b1 && b2) {
					place[rev[b1]] = -1;
					rev[b1] = -1;
				}
				for (u = 1; u < b1; u++) {
					place[rev[u]] = -1;
					rev[u] = -1;
				}
				delete w;
			}
			while (w = list2[6-ord]) {
				list2[6-ord] = w->next;
				delete w;
			}
		}

		delete temp;
		delete[] b;
		delete[] p1;
        DestroyGraph(g);
        delete[] rev;
        delete[] place;
    }
}

void ReadGraphs2ec(TVertex max_ord = 10) {
    FILE * in_file;
    char graph_text[100];
    char file_name[20];
    char * c;
    int i, g_order;
    TVertex u, v, bit, number;
	//int e_count;

	// zmiany
	for (i = 1, g_order = 9; g_order <= max_ord; g_order++) {
        sprintf(file_name, "%d_2ec.txt", g_order);
        in_file = fopen(file_name, "rt");
		//printf("\norder: %d\n", g_order);
        for ( ; !feof(in_file); i++)
        {
            // czytamy graf - format Brandana McKay'a; ilo wierzchokw to g_order
            fscanf(in_file, "%s\n", graph_text);
            c = graph_text+1;

            graphs2ec->table[i] = new TGraphListEl;
            graphs2ec->table[i]->next = NULL;
            graphs2ec->table[i]->graph = CreateGraph(g_order);
            for (u = 0; u < g_order; u++) {
                graphs2ec->table[i]->graph.d1[u] = u;
                graphs2ec->table[i]->graph.d2[u] = g_order-1-u;
                for (v = u+1; v < g_order; v++)
                    SetEdgeStatus(graphs2ec->table[i]->graph.e_arr, u, v, 2);
            }
            bit = 5;
            number = *c-63;
            c++;
		
			//e_count = 0;
            for (u = 1 ; u < g_order ; u++)
            {
                for (v = 0 ; v < u ; v++)
                {
                    if (bit == -1)
                    {
                        bit = 5;
                        number = *c-63;
                        c++;
                    }

                    if ((number >> bit) & 1) {
                        graphs2ec->table[i]->graph.d1[u]--;
                        graphs2ec->table[i]->graph.d1[v]++;
                        graphs2ec->table[i]->graph.d2[u]++;
                        graphs2ec->table[i]->graph.d2[v]--;
                        SetEdgeStatus(graphs2ec->table[i]->graph.e_arr, v, u, 1);
						//e_count++;
                    }
                    bit--;
                }
            }
			//printf("%d\t", e_count);
        }
        fclose(in_file);
    }
}

void ReadGraphs1ec(TVertex max_ord = 6) {
    FILE * in_file;
    char graph_text[100];
    char file_name[20];
    char * c;
    int i, g_order;
    TVertex u, v, bit, number;

	//zmiany
    for (i = 0, g_order = 3; g_order <= max_ord; g_order++) {
        sprintf(file_name, "%d_1ec.txt", g_order);
        in_file = fopen(file_name, "rt");
        for ( ; !feof(in_file); i++)
        {
            // czytamy graf - format Brandana McKay'a; ilo wierzchokw to g_order
            fscanf(in_file, "%s\n", graph_text);
            c = graph_text+1;

            graphs1ec.table[i] = new TGraphListEl;
            graphs1ec.table[i]->next = NULL;
            graphs1ec.table[i]->graph = CreateGraph(g_order);
            for (u = 0; u < g_order; u++) {
                graphs1ec.table[i]->graph.d1[u] = u;
                graphs1ec.table[i]->graph.d2[u] = g_order-1-u;
                for (v = u+1; v < g_order; v++)
                    SetEdgeStatus(graphs1ec.table[i]->graph.e_arr, u, v, 2);
            }
            bit = 5;
            number = *c-63;
            c++;

            for (u = 1 ; u < g_order ; u++)
            {
                for (v = 0 ; v < u ; v++)
                {
                    if (bit == -1)
                    {
                        bit = 5;
                        number = *c-63;
                        c++;
                    }

                    if ((number >> bit) & 1) {
                        graphs1ec.table[i]->graph.d1[u]--;
                        graphs1ec.table[i]->graph.d1[v]++;
                        graphs1ec.table[i]->graph.d2[u]++;
                        graphs1ec.table[i]->graph.d2[v]--;
                        SetEdgeStatus(graphs1ec.table[i]->graph.e_arr, v, u, 1);
                    }
                    bit--;
                }
            }
        }
        fclose(in_file);
    }
}

TShortGraphListEl * ReadShortGraphs(TVertex g_order, unsigned int nr) {
    FILE * in_file;
	FILE * out_file;
    char graph_text[30];
    char file_name[50];
    char * c;
    TVertex u, v, bit, number;
	TShortGraphListEl *w, *list = NULL;

    sprintf(file_name, "g%d_2ec\\g%d_2ec%d.txt", g_order, g_order, nr);
    in_file = fopen(file_name, "rt");
	sprintf(file_name, "g%d_2ec.txt", g_order);
	out_file = fopen(file_name, "at");
	graph_text[0] = 0;
    while (!feof(in_file)) {
		// czytamy graf - format Brandana McKay'a; ilo wierzchokw to g_order
        fscanf(in_file, "%s\n", graph_text);
		if (!graph_text[0]) break;  // problem z plikiem zerowej dugoci
		fprintf(out_file, "%s\n", graph_text);
        c = graph_text+1;

        w = list;
		list = new TShortGraphListEl;
		list->next = w;
		memset(list->e_arr, 0, short_size * sizeof(unsigned long));

        for (u = 0; u < g_order; u++)
			for (v = u+1; v < g_order; v++)
				SetEdgeStatus(list->e_arr, u, v, 2);

		bit = 5;
        number = *c-63;
        c++;

        for (u = 1 ; u < g_order ; u++) {
			for (v = 0 ; v < u ; v++) {
				if (bit == -1) {
					bit = 5;
                    number = *c-63;
                    c++;
                }
				if ((number >> bit) & 1)
					SetEdgeStatus(list->e_arr, v, u, 1);
                
                bit--;
            }
        }
    }
	//fprintf(out_file, "A\n"); // znacznik koca
	fclose(out_file);
    fclose(in_file);
	return list;
}

void ReadShortGraphs(const TVertex g_order, TShortGraphTreeNode ** graphs, int * count, int g_start = 0, int g_stop = 0) {
	// zmiany - turnieje
	unsigned int adr[5] = {0, 1, 2, 8, 44};
	unsigned int adr2[5];
    FILE * in_file;
    char graph_text[30];
	char file_name[30];
    char * c;
    TVertex i, u, v, ord, bit, number, g_num;
	TShortGraphTreeNode *w, *w_pop;
	TEdgePointer e_arr = new unsigned long[short_size];

	for(ord = 3; ord < g_order - 3; ord++)
		adr2[ord-3] = (adr[ord-2]-adr[ord-3])*(adr[g_order-ord-3]-adr[g_order-ord-4]);
	for(ord = 4; ord < g_order - 3; ord++)
		adr2[ord-3] += adr2[ord-4];
	
	for(g_num = g_start; g_num <= g_stop; g_num++) {
		if (!g_num) sprintf(file_name, "g%d_2ec.txt", g_order);
		else sprintf(file_name, "g%d_2ec%d.txt", g_order, g_num);
		in_file = fopen(file_name, "rt");
		//graphs[0] = NULL;
		//count[0] = 0;
		ord = 3; i = 0;
		while (!feof(in_file)) {
			// czytamy graf - format Brandana McKay'a; ilo wierzchokw to g_order
			fscanf(in_file, "%s\n", graph_text);
			if (graph_text[0]=='A') { // pusty napis - nowa grupa grafw
				i++;
				//graphs[i] = NULL;
				//count[i] = 0;
				if (i == adr2[ord-3]) ord++;
			}
			else {
				c = graph_text+1;

				for (u = 0; u < g_order; u++)
					for (v = u+1; v < g_order; v++)
						SetEdgeStatus(e_arr, u, v, 2);

				bit = 5;
				number = *c-63;
				c++;

				for (u = 1 ; u < g_order ; u++) {
					for (v = 0 ; v < u ; v++) {
						if (bit == -1) {
							bit = 5;
							number = *c-63;
							c++;
						}
						if ((number >> bit) & 1)
							SetEdgeStatus(e_arr, v, u, 1);
		                
						bit--;
					}
				}

				if(!count[i]++) {
					graphs[i] = new TShortGraphTreeNode;
					graphs[i]->p1 = graphs[i]->p2 = NULL;
					tree_count++;
				}
				w = graphs[i];
				//dopki jest cieka w drzewie
				for (u = 1; (u <= ord); u++) {
					for (v = ord+1; (v < g_order); v++) {
						w_pop = w;
						if (GetEdgeStatus(e_arr, u, v) == 1)
							w = w->p1;
						else w = w->p2;
						if (!w) break;
					}
					if (!w) break;
				}
				//nie ma cieki - tworzymy
				for ( ; (u <= ord); u++, v=ord+1)
					for ( ; (v < g_order); v++) {
						w = new TShortGraphTreeNode;
						w->p1 = w->p2 = NULL;
						tree_count++;
						if (GetEdgeStatus(e_arr, u, v) == 1)
							w_pop->p1 = w;
						else w_pop->p2 = w;
						w_pop = w;
					}
			}
		}
		fclose(in_file);
	}
	delete [] e_arr;
}

void DeleteShortGraphList(TShortGraphListEl ** list, unsigned int number) {
	unsigned int i;
	TShortGraphListEl * w;
	for (i = 0; i < number; i++)
		while (w = list[i]) {
			list[i] = w->next;
			delete w;
		}
}

void DeleteShortGraphTree(TShortGraphTreeNode *t) {
	if (t->p1) DeleteShortGraphTree(t->p1);
	if (t->p2) DeleteShortGraphTree(t->p2);
	delete t;
}

void DeleteShortGraphTreeList(TShortGraphTreeNode ** list, unsigned int number) {
	unsigned int i;
	for (i = 0; i < number; i++)
		if (list[i]) DeleteShortGraphTree(list[i]);
}

void ConstructGraphs(const TVertex g_order, THashTable * graphs, TShortGraphTreeNode ** graphs_2ec, int * count, int g_start = 0, int g_stop = 0) {
	// zmiany - jak bd obsadza 10-ki to si przyda
	unsigned int adr[5] = {0, 1, 2, 8, 44};
	unsigned int i = 0;
	unsigned int j, k;
	TVertex u, ord;
	TGraph g0, g1 = CreateGraph();
	TVertex p[11]= {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10};
	//THashTable *temp;
	//char s[100];

	ReadShortGraphs(g_order, graphs_2ec, count, g_start, g_stop);
	for (ord = 3; ord <= g_order-4; ord++) { 
		g0 = CreateGraph();
		for (u = 1; u <= ord; u++) {
			SetEdgeStatus(g0.e_arr, 0, u, 1);
			g0.d2[u] = 1;
		}
		g0.d1[0] = ord;
		for (u = g_order-1; u > ord; u--) {
			SetEdgeStatus(g0.e_arr, 0, u, 2);
			g0.d1[u] = 1;
		}
		g0.d2[0] = g_order-1-ord;
		for (j = adr[ord-3]; j < adr[ord-2]; j++) {
			CopyGraph(g0, g1);
			EmbedGraph(graphs1ec.table[j]->graph, g1, ord, p+1);
			for (k = adr[g_order-4-ord]; k < adr[g_order-3-ord]; k++) {
				// najpierw zaczytanie danych z pliku
				//graphs_2ec[i] = ReadShortGraphs(g_order, i);
				if (graphs_2ec[i]) {
					graphs->table[i] = new TGraphListEl;
					graphs->table[i]->next = NULL;
					graphs->table[i]->graph = GraphCopy(g1);
					EmbedGraph(graphs1ec.table[k]->graph, graphs->table[i]->graph, g_order-1-ord, p+ord+1);
				}
				i++;
				/*if (g_stop) {
					temp = new THashTable(19999);
					switch (g_order) {
						case 10: EmbedIsoGraph10(graphs->table[i-1]->graph, 10, 0, 0, 10, 0, p, temp, g_start, g_stop);
								 break;
						case 11: //EmbedIsoGraph11(graphs->table[i-1]->graph, 11, 0, 0, 11, 0, p, temp, g_start, g_stop);
								 break;
					}
					printf("%d - %d\n", i-1, temp->data_count);
					sprintf(s, "g%d_2ec%d.txt", g_order, i-1);
					temp->WriteReport(s);
					delete temp;
				}//*/
			}
		}
		DestroyGraph(g0);
	}
	DestroyGraph(g1);
}

void SelectVertices(TVertex vert, TVertex v_stop, TVertex v_count) {
	// wybiera (wyrnia) v_count wierzchokw spord vert..v_stop
	// pomyle nad inn koncepcj - wybiera tylko wierzchoki o ktrych co wiadomo
	THashTable src(1), dest(1);
	TPermPointer p1 = new TVertex[order];
	TVertex u;
	TGraphListEl *w;
	
	if (!v_stop) v_stop = order-1;

	dest.fname = fname_out;
	src.ReadGraphList(fname_in);

	// ustali kombinacj
	for (u = 0; u < v_count; u++)
		p1[u] = vert+u;
	while (p1[0] <= v_stop + 1 - v_count) {
		// przeksztaci graf i dopisa do dest
		for (w = src.table[0]; w; w = w->next) {
			for (u = 0; u < v_count; u++)
				w->graph.d1[p1[u]] += 2*order;
			dest.InsertGraphWC(w->graph, 0);
			for (u = 0; u < v_count; u++)
				w->graph.d1[p1[u]] -= 2*order;
		}

		// wzi nastpn kombinacj
		if (p1[v_count-1] >= v_stop) {
			u = v_count - 2;
			while (u > 0 && p1[u] == p1[u+1] - 1)
				u--;
			if (u < 0) u = 0;
			p1[u]++;
			for (u++; u < v_count; u++)
				p1[u] = p1[u-1]+1;
		}
		else p1[v_count-1]++;
	}
	delete[] p1;

	if (dest.data_count) dest.WriteGraphList(fname_out);
}
